Package: PriceIndices
Title: Calculating Bilateral and Multilateral Price Indexes
Version: 0.3.1
Authors@R: 
    person(given = "Jacek",
           family = "Białek",
           role = c("aut", "cre"),
           email = "jacek.bialek@uni.lodz.pl",
           comment = c(ORCID = "0000-0002-0952-5327"))
Description: Preparing a scanner data set for price dynamics calculations (data selecting, data classification, data matching, data filtering). Computing bilateral and multilateral indexes. For details on these methods see: Diewert and Fox (2020) 
    <doi:10.1080/07350015.2020.1816176>, Białek (2019) <doi:10.2478/jos-2019-0014> or Białek (2020) <doi:10.2478/jos-2020-0037>.
Depends: R (>= 3.5.0)
Imports: lubridate (>= 1.7.4), dplyr (>= 0.8.3), ggplot2 (>= 3.2.0),
        reshape, reclin2, stringr, strex
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: testthat, knitr, rmarkdown
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-18 12:30:24 UTC; User
Author: Jacek Białek [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0952-5327>)
Maintainer: Jacek Białek <jacek.bialek@uni.lodz.pl>
Repository: CRAN
Date/Publication: 2026-04-18 12:50:08 UTC
Built: R 4.5.2; ; 2026-04-20 23:33:24 UTC; unix
