# Generated by roxygen2: do not edit by hand

export(IQMp)
export(MARS)
export(QMp)
export(QMq)
export(QU)
export(SPQ)
export(agmean)
export(available)
export(banajree)
export(bennet)
export(bialek)
export(bmw)
export(carli)
export(ccdi)
export(ccdi_fbew)
export(ccdi_fbmw)
export(ccdi_splice)
export(chIQMp)
export(chQMp)
export(chQMq)
export(chagmean)
export(chbanajree)
export(chbialek)
export(chbmw)
export(chcarli)
export(chcswd)
export(chdavies)
export(chdikhanov)
export(chdrobisch)
export(chdutot)
export(chfisher)
export(chgeary_khamis)
export(chgeohybrid)
export(chgeolaspeyres)
export(chgeolowe)
export(chgeopaasche)
export(chgeowalsh)
export(chgeoyoung)
export(chharmonic)
export(chhlc)
export(chhybrid)
export(chjevons)
export(chlaspeyres)
export(chlehr)
export(chlloyd_moulton)
export(chlowe)
export(chmarshall_edgeworth)
export(chpaasche)
export(chpalgrave)
export(chsato_vartia)
export(chstuvel)
export(chtheil1)
export(chtheil2)
export(chtornqvist)
export(chvartia)
export(chwalsh)
export(chwalsh_vartia)
export(chybmd)
export(chyoung)
export(compare_distances)
export(compare_indices_df)
export(compare_indices_jk)
export(compare_indices_list)
export(compare_to_target)
export(cswd)
export(data_aggregating)
export(data_check)
export(data_filtering)
export(data_imputing)
export(data_matching)
export(data_norm)
export(data_preparing)
export(data_reducing)
export(data_selecting)
export(data_unit)
export(davies)
export(dikhanov)
export(dissimilarity)
export(dissimilarity_fig)
export(drobisch)
export(dutot)
export(elasticity)
export(elasticity_fig)
export(expenditures)
export(final_index)
export(fisher)
export(geary_khamis)
export(geks)
export(geks_fbew)
export(geks_fbmw)
export(geks_splice)
export(geksaqi)
export(geksaqi_fbew)
export(geksaqi_fbmw)
export(geksaqi_splice)
export(geksaqu)
export(geksaqu_fbew)
export(geksaqu_fbmw)
export(geksaqu_splice)
export(geksgaqi)
export(geksgaqi_fbew)
export(geksgaqi_fbmw)
export(geksgaqi_splice)
export(geksgl)
export(geksgl_fbew)
export(geksgl_fbmw)
export(geksgl_splice)
export(geksiqm)
export(geksiqm_fbew)
export(geksiqm_fbmw)
export(geksiqm_splice)
export(geksj)
export(geksj_fbew)
export(geksj_fbmw)
export(geksj_splice)
export(geksl)
export(geksl_fbew)
export(geksl_fbmw)
export(geksl_splice)
export(gekslm)
export(gekslm_fbew)
export(gekslm_fbmw)
export(gekslm_splice)
export(geksqm)
export(geksqm_fbew)
export(geksqm_fbmw)
export(geksqm_splice)
export(geksw)
export(geksw_fbew)
export(geksw_fbmw)
export(geksw_splice)
export(generate)
export(generate_CES)
export(geohybrid)
export(geolaspeyres)
export(geolowe)
export(geopaasche)
export(geowalsh)
export(geoyoung)
export(gk)
export(gk_fbew)
export(gk_fbmw)
export(gk_splice)
export(harmonic)
export(hlc)
export(hybrid)
export(jevons)
export(laspeyres)
export(lehr)
export(lloyd_moulton)
export(lowe)
export(m_decomposition)
export(marshall_edgeworth)
export(matched)
export(matched_fig)
export(matched_index)
export(mbennet)
export(mmontgomery)
export(montgomery)
export(outlet_check)
export(paasche)
export(palgrave)
export(pqcor)
export(pqcor_fig)
export(price_indices)
export(prices)
export(products)
export(products_fig)
export(quantities)
export(resultant_index)
export(retro_index)
export(sales)
export(sales_groups)
export(sales_groups2)
export(sato_vartia)
export(shrinkflation)
export(stuvel)
export(theil1)
export(theil2)
export(tindex)
export(tornqvist)
export(tpd)
export(tpd_fbew)
export(tpd_fbmw)
export(tpd_splice)
export(unit_value_index)
export(utpd)
export(utpd_fbew)
export(utpd_fbmw)
export(utpd_splice)
export(value_index)
export(vartia)
export(walsh)
export(walsh_vartia)
export(wgeks)
export(wgeks_fbew)
export(wgeks_fbmw)
export(wgeks_splice)
export(wgeksaqi)
export(wgeksaqi_fbew)
export(wgeksaqi_fbmw)
export(wgeksaqi_splice)
export(wgeksaqu)
export(wgeksaqu_fbew)
export(wgeksaqu_fbmw)
export(wgeksaqu_splice)
export(wgeksgaqi)
export(wgeksgaqi_fbew)
export(wgeksgaqi_fbmw)
export(wgeksgaqi_splice)
export(wgeksgl)
export(wgeksgl_fbew)
export(wgeksgl_fbmw)
export(wgeksgl_splice)
export(wgeksl)
export(wgeksl_fbew)
export(wgeksl_fbmw)
export(wgeksl_splice)
export(ybmd)
export(young)
