#' An artificial data set on sold coffee
#'
#' A collection of scanner data on the sale of coffee in the period from January 2024 to February 2024
#' @format A data frame with 6 columns and 51 rows. The used variables are as follows:
#'
#' \code{time} - Dates of transactions (Year-Month-Day)
#'
#' \code{prices} - Prices of sold products [PLN]
#'
#' \code{quantities} - Quantities of sold products [liters]
#'
#' \code{codeIN} - Unique internal product codes (retaler product codes)
#'
#' \code{codeOUT} - Unique external product codes (e.g. GTIN, EAN, SKU)
#'
#' \code{description} - Descriptions of sold coffee products 
#' @docType data
#'
"data_DOWN_UP_SIZED"