% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_DOWN_UP_SIZED.R
\docType{data}
\name{data_DOWN_UP_SIZED}
\alias{data_DOWN_UP_SIZED}
\title{An artificial data set on sold coffee}
\format{
A data frame with 6 columns and 51 rows. The used variables are as follows:

\code{time} - Dates of transactions (Year-Month-Day)

\code{prices} - Prices of sold products [PLN]

\code{quantities} - Quantities of sold products [liters]

\code{codeIN} - Unique internal product codes (retaler product codes)

\code{codeOUT} - Unique external product codes (e.g. GTIN, EAN, SKU)

\code{description} - Descriptions of sold coffee products
}
\usage{
data_DOWN_UP_SIZED
}
\description{
A collection of scanner data on the sale of coffee in the period from January 2024 to February 2024
}
\keyword{datasets}
